@@ -1,7 +1,21 @@
-from typing import Generator, Any, AsyncGenerator, Optional, Mapping, Union, Sequence, Type, List, Callable, TypeVar
+from contextvars import ContextVar
+from threading import Lock
+from typing import (
+    Any,
+    AsyncGenerator,
+    Callable,
+    Generator,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Type,
+    TypeVar,
+    Union,
+)
 
-from sqlalchemy.engine import Result, Connection
-from sqlalchemy.ext.asyncio import AsyncEngine, create_async_engine, AsyncConnection
+from sqlalchemy.engine import Connection, Result
+from sqlalchemy.ext.asyncio import AsyncConnection, AsyncEngine, create_async_engine
 from sqlalchemy.future import Engine, create_engine
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.sql import Executable, Select
@@ -23,9 +37,9 @@ _R = TypeVar("_R")
 _ExecuteParams = Union[Mapping[Any, Any], Sequence[Mapping[Any, Any]]]
 _ExecuteOptions = Mapping[Any, Any]
 
+
 class AsyncDatabase(AbcAsyncDatabase):
-    """`sqlalchemy` asynchronous database client
-    """
+    """`sqlalchemy` asynchronous database client"""
 
     def __init__(self, engine: AsyncEngine, **session_options):
         """
@@ -37,17 +51,17 @@ class AsyncDatabase(AbcAsyncDatabase):
         super().__init__()
         self.engine: AsyncEngine = engine
         """`sqlalchemy` Asynchronous Engine
-        
+
         Example:
             ```Python
             async with self.engine.begin() as conn:
                 await conn.run_sync(SQLModel.metadata.create_all)
             ```
         """
-        session_options.setdefault('class_', AsyncSession)
+        session_options.setdefault("class_", AsyncSession)
         self.session_maker: Callable[..., AsyncSession] = sessionmaker(self.engine, **session_options)
         """`sqlalchemy` session factory function
-        
+
         Example:
             ```Python
             async with self.session_maker() as sesson:
@@ -55,6 +69,50 @@ class AsyncDatabase(AbcAsyncDatabase):
                 await session.commit()
             ```
         """
+        self._session_lock = Lock()
+        self._session_enter_count = 0
+        self._session_context_var: ContextVar[Optional[AsyncSession]] = ContextVar("_session_context_var", default=None)
+
+    @property
+    def session(self) -> Optional[AsyncSession]:
+        """Return an instance of Session local to the current async context.
+
+        Note: Must register middleware in fastapi application to get session in request.
+
+        Example:
+            ```Python
+            app = FastAPI()
+            app.add_middleware(BaseHTTPMiddleware, dispatch=db.asgi_dispatch)
+
+            @app.get('/get_user')
+            async def get_user(id:int):
+                return await db.session.get(User,id)
+            ```
+        In ordinary methods, session will return None. You can get it through:
+            ```Python
+            async with db:
+                db.session.get(User,id)
+            ```
+        """
+        return self._session_context_var.get() if self._session_enter_count > 0 else None
+
+    async def __aenter__(self):
+        with self._session_lock:
+            session = self.session
+            if session is None:
+                session = self.session_maker()
+                self._session_context_var_token = self._session_context_var.set(session)
+            self._session_enter_count += 1
+        return session
+
+    async def __aexit__(self, exc_type, exc_value, traceback):
+        with self._session_lock:
+            self._session_enter_count -= 1
+            if self._session_enter_count <= 0:
+                session = self._session_context_var.get()
+                if session is not None:
+                    await session.close()
+                self._session_context_var.reset(self._session_context_var_token)
 
     @classmethod
     def create(cls, url: str, *, session_options: Mapping[str, Any] = None, **kwargs) -> "AsyncDatabase":
@@ -68,7 +126,7 @@ class AsyncDatabase(AbcAsyncDatabase):
         Returns:
             Return the client instance.
         """
-        kwargs.setdefault('future', True)
+        kwargs.setdefault("future", True)
         engine = create_async_engine(url, **kwargs)
         session_options = session_options or {}
         return cls(engine, **session_options)
@@ -139,18 +197,23 @@ class AsyncDatabase(AbcAsyncDatabase):
         Note:
             After the connection is closed, the session data will be cleared.
             Please use the 'on_close_pre' parameter to process the execution result.
-            example: The `Result.all`,`Result.scalar`,`Result.scalars`methods cannot be called again after the connection is closed.
+            example: The `Result.all`,`Result.scalar`,`Result.scalars`methods
+                cannot be called again after the connection is closed.
 
         """
         need_close = False
         if executor is None or not isinstance(executor, (AsyncSession, AsyncConnection)):
             need_close = True
             if is_session:
-                executor = self.session_maker()
-                kw['bind_arguments'] = bind_arguments
+                executor = self.session
+                if executor is None:
+                    executor = self.session_maker()
+                else:
+                    need_close = False
+                kw["bind_arguments"] = bind_arguments
             else:
                 executor = await self.engine.connect()
-        async with ExecutorContextManager(executor, need_close = need_close) as executor:
+        async with ExecutorContextManager(executor, need_close=need_close) as executor:
             result = await executor.execute(statement, params, execution_options, **kw)  # type:ignore
             if on_close_pre:
                 result = on_close_pre(result)
@@ -174,16 +237,18 @@ class AsyncDatabase(AbcAsyncDatabase):
         Usage and parameters are the same as that of :meth:`_orm.Session.execute`;
         the return result is a scalar Python value.
         """
+        need_close = False
         if session is None or not isinstance(session, AsyncSession):
-            executor = self.session_maker()
-        else:
-            executor = session
-        async with ExecutorContextManager(executor, need_close = session is None) as executor:
-            result = await executor.scalar(
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        async with ExecutorContextManager(session, need_close=need_close) as session:
+            result = await session.scalar(
                 statement,
                 params,
-                execution_options = execution_options,
-                bind_arguments = bind_arguments,
+                execution_options=execution_options,
+                bind_arguments=bind_arguments,
                 **kw,
             )
             return result
@@ -203,17 +268,21 @@ class AsyncDatabase(AbcAsyncDatabase):
         Usage and parameters are the same as that of :meth:`_orm.Session.execute`;
         the return result is a list of scalar Python value.
         """
+        need_close = False
         if session is None or not isinstance(session, AsyncSession):
-            executor = self.session_maker()
-        else:
-            executor = session
-        async with ExecutorContextManager(executor, need_close = session is None) as executor:
-            result = (await executor.scalars(
-                statement,
-                params,
-                execution_options = execution_options,
-                **kw,
-            )).all()
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        async with ExecutorContextManager(session, need_close=need_close) as session:
+            result = (
+                await session.scalars(
+                    statement,
+                    params,
+                    execution_options=execution_options,
+                    **kw,
+                )
+            ).all()
             return result
 
     async def get(
@@ -268,33 +337,34 @@ class AsyncDatabase(AbcAsyncDatabase):
             )
             ```
         """
+        need_close = False
         if session is None or not isinstance(session, AsyncSession):
-            executor = self.session_maker()
-        else:
-            executor = session
-        async with ExecutorContextManager(executor, need_close = session is None) as executor:
-            result = await executor.get(
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        async with ExecutorContextManager(session, need_close=need_close) as session:
+            result = await session.get(
                 entity,
                 ident,
-                options = options,
-                populate_existing = populate_existing,
-                with_for_update = with_for_update,
-                identity_token = identity_token,
+                options=options,
+                populate_existing=populate_existing,
+                with_for_update=with_for_update,
+                identity_token=identity_token,
             )
             return result
 
     async def delete(self, instance: Any) -> None:
         """Deletes an instance object."""
-        async with self.session_maker() as session:
-            async with session.begin():
-                await session.delete(instance)
+        if self.session is not None:
+            await self.session.delete(instance)
+            await self.session.commit()
+        else:
+            async with self.session_maker() as session:
+                async with session.begin():
+                    await session.delete(instance)
 
-    async def save(
-        self,
-        *instances: Any,
-        refresh: bool = False,
-        session: Optional[AsyncSession] = None
-    ) -> None:
+    async def save(self, *instances: Any, refresh: bool = False, session: Optional[AsyncSession] = None) -> None:
         """
         Save the given collection of instances.
             *instances: A sequence of instance objects.
@@ -302,15 +372,17 @@ class AsyncDatabase(AbcAsyncDatabase):
                     Args:
             session: If not specified, an `AsyncSession` is created.
         """
+        need_close = False
         if session is None or not isinstance(session, AsyncSession):
-            executor = self.session_maker()
-        else:
-            executor = session
-        async with ExecutorContextManager(executor, need_close = session is None) as executor:
-            executor.add_all(instances)
-            await executor.commit()
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        async with ExecutorContextManager(session, need_close=need_close) as session:
+            session.add_all(instances)
+            await session.commit()
             if refresh:
-                [await executor.refresh(instance) for instance in instances]
+                [await session.refresh(instance) for instance in instances]
 
     async def run_sync(
         self,
@@ -320,7 +392,7 @@ class AsyncDatabase(AbcAsyncDatabase):
         on_close_pre: Callable[[_T], _R] = None,
         is_session: bool = True,
         executor: Union[AsyncSession, AsyncConnection, None] = None,
-        **kwargs: _P.kwargs
+        **kwargs: _P.kwargs,
     ) -> Union[_T, _R]:
         """
         Invoke the given sync callable passing sync self as the first
@@ -356,10 +428,18 @@ class AsyncDatabase(AbcAsyncDatabase):
             APIs which will be properly adapted to the greenlet context.
         """
         need_close = False
+        if executor is None and is_session:
+            executor = self.session
         if executor is None or not isinstance(executor, (AsyncSession, AsyncConnection)):
-            need_close = True
-            executor = self.session_maker() if is_session else await self.engine.connect()
-        async with ExecutorContextManager(executor, need_close = need_close) as executor:
+            if is_session:
+                executor = self.session
+                if executor is None:
+                    executor = self.session_maker()
+                    need_close = True
+            else:
+                executor = await self.engine.connect()
+                need_close = True
+        async with ExecutorContextManager(executor, need_close=need_close) as executor:
             result = await executor.run_sync(fn, *args, **kwargs)
             if on_close_pre:
                 result = on_close_pre(result)
@@ -367,20 +447,51 @@ class AsyncDatabase(AbcAsyncDatabase):
                 await executor.commit()
             return result
 
-class Database(AbcAsyncDatabase):
-    """`sqlalchemy` synchronous database client
 
-    """
+class Database(AbcAsyncDatabase):
+    """`sqlalchemy` synchronous database client"""
 
     def __init__(self, engine: Engine, **session_options):
         super().__init__()
         self.engine: Engine = engine
-        session_options.setdefault('class_', Session)
+        session_options.setdefault("class_", Session)
         self.session_maker: Callable[..., Session] = sessionmaker(self.engine, **session_options)
+        self._session_lock = Lock()
+        self._session_enter_count = 0
+        self._session_context_var: ContextVar[Optional[Session]] = ContextVar("_session_context_var", default=None)
+
+    @property
+    def session(self) -> Optional[Session]:
+        """Return an instance of Session local to the current context."""
+        return self._session_context_var.get() if self._session_enter_count > 0 else None
+
+    def __enter__(self):
+        with self._session_lock:
+            session = self.session
+            if session is None:
+                session = self.session_maker()
+                self._session_context_var_token = self._session_context_var.set(session)
+            self._session_enter_count += 1
+        return session
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        with self._session_lock:
+            self._session_enter_count -= 1
+            if self._session_enter_count <= 0:
+                session = self._session_context_var.get()
+                if session is not None:
+                    session.close()
+                self._session_context_var.reset(self._session_context_var_token)
+
+    async def __aenter__(self):
+        return self.__enter__()
+
+    async def __aexit__(self, exc_type, exc_value, traceback):
+        return self.__exit__(exc_type, exc_value, traceback)
 
     @classmethod
     def create(cls, url: str, *, session_options: Optional[Mapping[str, Any]] = None, **kwargs) -> "Database":
-        kwargs.setdefault('future', True)
+        kwargs.setdefault("future", True)
         engine = create_engine(url, **kwargs)
         session_options = session_options or {}
         return cls(engine, **session_options)
@@ -403,14 +514,20 @@ class Database(AbcAsyncDatabase):
         **kw: Any,
     ) -> Union[Result, _T]:
         need_close = False
+        if executor is None and is_session:
+            executor = self.session
         if executor is None or not isinstance(executor, (Session, Connection)):
             need_close = True
             if is_session:
-                executor = self.session_maker()
-                kw['bind_arguments'] = bind_arguments
+                executor = self.session
+                if executor is None:
+                    executor = self.session_maker()
+                else:
+                    need_close = False
+                kw["bind_arguments"] = bind_arguments
             else:
                 executor = self.engine.connect()
-        with ExecutorContextManager(executor, need_close = need_close) as executor:
+        with ExecutorContextManager(executor, need_close=need_close) as executor:
             result = executor.execute(statement, params, execution_options, **kw)
             if on_close_pre:
                 result = on_close_pre(result)
@@ -428,16 +545,18 @@ class Database(AbcAsyncDatabase):
         session: Optional[Session] = None,
         **kw: Any,
     ) -> Any:
+        need_close = False
         if session is None or not isinstance(session, Session):
-            executor = self.session_maker()
-        else:
-            executor = session
-        with ExecutorContextManager(executor, need_close = session is None) as executor:
-            result = executor.scalar(
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        with ExecutorContextManager(session, need_close=need_close) as session:
+            result = session.scalar(
                 statement,
                 params,
-                execution_options = execution_options,
-                bind_arguments = bind_arguments,
+                execution_options=execution_options,
+                bind_arguments=bind_arguments,
                 **kw,
             )
             return result
@@ -452,16 +571,18 @@ class Database(AbcAsyncDatabase):
         session: Optional[Session] = None,
         **kw: Any,
     ) -> List[Any]:
+        need_close = False
         if session is None or not isinstance(session, Session):
-            executor = self.session_maker()
-        else:
-            executor = session
-        with ExecutorContextManager(executor, need_close = session is None) as executor:
-            result = executor.scalars(
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        with ExecutorContextManager(session, need_close=need_close) as session:
+            result = session.scalars(
                 statement,
                 params,
-                execution_options = execution_options,
-                bind_arguments = bind_arguments,
+                execution_options=execution_options,
+                bind_arguments=bind_arguments,
                 **kw,
             ).all()
             return result
@@ -478,41 +599,44 @@ class Database(AbcAsyncDatabase):
         execution_options: Optional[_ExecuteOptions] = None,
         session: Optional[Session] = None,
     ) -> Optional[_T]:
+        need_close = False
         if session is None or not isinstance(session, Session):
-            executor = self.session_maker()
-        else:
-            executor = session
-        with ExecutorContextManager(executor, need_close = session is None) as executor:
-            result = executor.get(
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        with ExecutorContextManager(session, need_close=need_close) as session:
+            result = session.get(
                 entity,
                 ident,
-                options = options,
-                populate_existing = populate_existing,
-                with_for_update = with_for_update,
-                identity_token = identity_token,
+                options=options,
+                populate_existing=populate_existing,
+                with_for_update=with_for_update,
+                identity_token=identity_token,
             )
             return result
 
     def delete(self, instance: Any) -> None:
-        with self.session_maker() as session:
-            with session.begin():
-                session.delete(instance)
-
-    def save(
-        self,
-        *instances: Any,
-        refresh: bool = False,
-        session: Optional[Session] = None
-    ) -> None:
-        if session is None or not isinstance(session, Session):
-            executor = self.session_maker()
+        if self.session is not None:
+            self.session.delete(instance)
+            self.session.commit()
         else:
-            executor = session
-        with ExecutorContextManager(executor, need_close = session is None) as executor:
-            executor.add_all(instances)
-            executor.commit()
+            with self.session_maker() as session:
+                with session.begin():
+                    session.delete(instance)
+
+    def save(self, *instances: Any, refresh: bool = False, session: Optional[Session] = None) -> None:
+        need_close = False
+        if session is None or not isinstance(session, Session):
+            session = self.session
+            if session is None:
+                need_close = True
+                session = self.session_maker()
+        with ExecutorContextManager(session, need_close=need_close) as session:
+            session.add_all(instances)
+            session.commit()
             if refresh:
-                [executor.refresh(instance) for instance in instances]
+                [session.refresh(instance) for instance in instances]
 
     def run_sync(
         self,
@@ -522,13 +646,20 @@ class Database(AbcAsyncDatabase):
         on_close_pre: Callable[[_T], _R] = None,
         is_session: bool = True,
         executor: Union[Session, Connection, None] = None,
-        **kwargs: _P.kwargs
+        **kwargs: _P.kwargs,
     ) -> Union[_T, _R]:
         need_close = False
         if executor is None or not isinstance(executor, (Session, Connection)):
             need_close = True
-            executor = self.session_maker() if is_session else self.engine.connect()
-        with ExecutorContextManager(executor, need_close = need_close) as executor:
+            if is_session:
+                executor = self.session
+                if executor is None:
+                    executor = self.session_maker()
+                else:
+                    need_close = False
+            else:
+                executor = self.engine.connect()
+        with ExecutorContextManager(executor, need_close=need_close) as executor:
             result = fn(executor, *args, **kwargs)
             if on_close_pre:
                 result = on_close_pre(result)
@@ -536,14 +667,11 @@ class Database(AbcAsyncDatabase):
                 executor.commit()
             return result
 
+
 class ExecutorContextManager:
     """Actuator context manager, optionally closing the executor"""
 
-    def __init__(
-        self,
-        executor: Union[Session, Connection, AsyncSession, AsyncConnection],
-        need_close: bool = True
-    ):
+    def __init__(self, executor: Union[Session, Connection, AsyncSession, AsyncConnection], need_close: bool = True):
         self.executor = executor
         self.need_close = need_close
 
